>From b864e0e01127185424f5c41a47dc1f96adbee65e Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?Jan=20Urba=C5=84ski?= <wulczer@wulczer.org>
Date: Sun, 2 Jan 2011 11:32:57 +0100
Subject: [PATCH 09/16] Call PLy_spi_execute_fetch_result inside the try/catch block.

This way errors from fetching tuples are correctly reported as errors
in the SPI call. While at it, ged rid of a useless loop variable, do
not pfree the result of a possible palloc(0) and do some minor
stylistic changes.
---
 src/pl/plpython/plpython.c |   49 +++++++++++++++++++++++++------------------
 1 files changed, 28 insertions(+), 21 deletions(-)

diff --git a/src/pl/plpython/plpython.c b/src/pl/plpython/plpython.c
index 38ff7f5..6de0365 100644
--- a/src/pl/plpython/plpython.c
+++ b/src/pl/plpython/plpython.c
@@ -2956,11 +2956,11 @@ PLy_spi_execute(PyObject *self, PyObject *args)
 static PyObject *
 PLy_spi_execute_plan(PyObject *ob, PyObject *list, long limit)
 {
-	volatile int nargs;
-	int			i,
-				rv;
-	PLyPlanObject *plan;
-	volatile MemoryContext oldcontext;
+	int 			volatile nargs;
+	int				i, rv;
+	PLyPlanObject	*plan;
+	PyObject		*ret;
+	MemoryContext volatile oldcontext = CurrentMemoryContext;
 
 	if (list != NULL)
 	{
@@ -2981,7 +2981,7 @@ PLy_spi_execute_plan(PyObject *ob, PyObject *list, long limit)
 		char	   *sv;
 		PyObject   *so = PyObject_Str(list);
 
-		if (!so)
+		if (so == NULL)
 			PLy_elog(ERROR, "could not execute plan");
 		sv = PyString_AsString(so);
 		PLy_exception_set_plural(PLy_exc_spi_error,
@@ -2994,11 +2994,15 @@ PLy_spi_execute_plan(PyObject *ob, PyObject *list, long limit)
 		return NULL;
 	}
 
-	oldcontext = CurrentMemoryContext;
 	PG_TRY();
 	{
-		char	   *nulls = palloc(nargs * sizeof(char));
-		volatile int j;
+		char	*nulls;
+		int		volatile j;
+
+		if (nargs > 0)
+			nulls = palloc(nargs * sizeof(char));
+		else
+			nulls = NULL;
 
 		for (j = 0; j < nargs; j++)
 		{
@@ -3038,12 +3042,15 @@ PLy_spi_execute_plan(PyObject *ob, PyObject *list, long limit)
 
 		rv = SPI_execute_plan(plan->plan, plan->values, nulls,
 							  PLy_curr_procedure->fn_readonly, limit);
+		ret = PLy_spi_execute_fetch_result(SPI_tuptable, SPI_processed, rv);
+
+		if (nargs > 0)
+			pfree(nulls);
+
 
-		pfree(nulls);
 	}
 	PG_CATCH();
 	{
-		int			k;
 		ErrorData	*edata;
 
 		MemoryContextSwitchTo(oldcontext);
@@ -3053,13 +3060,13 @@ PLy_spi_execute_plan(PyObject *ob, PyObject *list, long limit)
 		/*
 		 * cleanup plan->values array
 		 */
-		for (k = 0; k < nargs; k++)
+		for (i = 0; i < nargs; i++)
 		{
-			if (!plan->args[k].out.d.typbyval &&
-				(plan->values[k] != PointerGetDatum(NULL)))
+			if (!plan->args[i].out.d.typbyval &&
+				(plan->values[i] != PointerGetDatum(NULL)))
 			{
-				pfree(DatumGetPointer(plan->values[k]));
-				plan->values[k] = PointerGetDatum(NULL);
+				pfree(DatumGetPointer(plan->values[i]));
+				plan->values[i] = PointerGetDatum(NULL);
 			}
 		}
 
@@ -3082,21 +3089,21 @@ PLy_spi_execute_plan(PyObject *ob, PyObject *list, long limit)
 		}
 	}
 
-
-	return PLy_spi_execute_fetch_result(SPI_tuptable, SPI_processed, rv);
+	return ret;
 }
 
 static PyObject *
 PLy_spi_execute_query(char *query, long limit)
 {
 	int			rv;
-	volatile MemoryContext oldcontext;
+	PyObject	*ret;
+	MemoryContext volatile oldcontext = CurrentMemoryContext;
 
-	oldcontext = CurrentMemoryContext;
 	PG_TRY();
 	{
 		pg_verifymbstr(query, strlen(query), false);
 		rv = SPI_execute(query, PLy_curr_procedure->fn_readonly, limit);
+		ret = PLy_spi_execute_fetch_result(SPI_tuptable, SPI_processed, rv);
 	}
 	PG_CATCH();
 	{
@@ -3122,7 +3129,7 @@ PLy_spi_execute_query(char *query, long limit)
 		return NULL;
 	}
 
-	return PLy_spi_execute_fetch_result(SPI_tuptable, SPI_processed, rv);
+	return ret;
 }
 
 static PyObject *
-- 
1.7.2.3

